using System;
using System.Collections.Generic;
using System.Linq;
using Abt.Controls.SciChart.Example.Common;
using Abt.Controls.SciChart.Example.Data;
using Abt.Controls.SciChart.Example.MVVM;

namespace Abt.Controls.SciChart.Example.Examples.IWantTo.SeeFeaturedApplication.SciTrader
{
    public class SciTraderViewModel : BaseViewModel, IExampleAware
    {        
        private readonly IEnumerable<ChartType> _allChartTypes = new[] { ChartType.FastLine, ChartType.FastColumn, ChartType.FastMountain, ChartType.FastCandlestick, ChartType.FastOhlc,  };
        private Instrument _instrument;
        private TimeFrame _timeFrame;
        private ChartType _chartType;
        private IRange _chartDateRange;
        private IDataManager _dataManager = DataManager.Instance;
        private bool _isShowing200Sma;
        private bool _isShowing50Sma;
        private IOhlcDataSeries<DateTime, double> _priceSeries;
        private IXyDataSeries<DateTime, double> _sma200Series;
        private IXyDataSeries<DateTime, double> _sma50Series;
        private IDataSeriesSet<DateTime, double> _priceDataSet;
        private IDataSeriesSet<DateTime, long> _volumeData;
        private ModifierType _chartModifier;

        public SciTraderViewModel()
        {
            SetDefaults();            
        }

        /// <summary>
        /// Overloaded constructor used for testing
        /// </summary>
        /// <param name="dataManager"></param>
        public SciTraderViewModel(IDataManager dataManager)
        {
            _dataManager = dataManager;
            SetDefaults();            
        }

        public ActionCommand SetCursorModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.CrosshairsCursor));}}
        public ActionCommand SetRolloverModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.Rollover)); } }
        public ActionCommand SetRubberBandZoomModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.RubberBandZoom)); } }
        public ActionCommand SetZoomPanModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.ZoomPan)); } }
        public ActionCommand SetNullModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.Null)); } }

        public IEnumerable<Instrument> AllInstruments { get { return _dataManager.AvailableInstruments; } }
        public IEnumerable<TimeFrame> AllTimeFrames { get { return _dataManager.GetAvailableTimeFrames(SelectedInstrument); } }
        public IEnumerable<ChartType> AllChartTypes { get { return _allChartTypes; } }

        public IDataSeriesSet<DateTime, double> PriceData
        {
            get { return _priceDataSet; }
            set 
            { 
                _priceDataSet = value;
                OnPropertyChanged("PriceData");
            }
        }

        public IDataSeriesSet<DateTime, long> VolumeData
        {
            get { return _volumeData; }
            set
            {
                _volumeData = value;
                OnPropertyChanged("VolumeData");
            }
        }

        public Instrument SelectedInstrument
        {
            get { return _instrument; }
            set
            {
                _instrument = value;                
                
                OnPropertyChanged("AllTimeFrames");
                OnPropertyChanged("SelectedInstrument");
                OnPropertyChanged("PriceChartTextFormatting");

                // Since timeframes depend on the instrument, get a new timeframe before updating the chart and below, raise PropertyChanged events
                SelectedTimeFrame = _dataManager.GetAvailableTimeFrames(_instrument).First();
            }
        }        

        public ChartType SelectedChartType
        {
            get { return _chartType; }
            set
            {
                _chartType = value;
                OnPropertyChanged("SelectedChartType");
            }
        }

        public TimeFrame SelectedTimeFrame
        {
            get { return _timeFrame; }
            set
            {
                if (value != null)
                {
                    _timeFrame = value;
                    UpdateChartData(_instrument, _timeFrame);
                    OnPropertyChanged("SelectedTimeFrame");
                }
            }
        }

        public IRange ChartVisibleRange
        {
            get { return _chartDateRange; }
            set
            {
                _chartDateRange = value;
                OnPropertyChanged("ChartVisibleRange");
            }
        }

        public bool Show200SMa
        {
            get { return _isShowing200Sma; }
            set
            {
                _isShowing200Sma = value;                
                OnPropertyChanged("IsShowing200Sma");
            }
        }


        public bool Show50Sma
        {
            get { return _isShowing50Sma; }
            set
            {
                _isShowing50Sma = value;
                OnPropertyChanged("IsShowing50Sma");
            }
        }

        /// <summary>
        /// Generates a formatting string for the Y-Axis, if DecimalPlaces = 1, generates "#", if DecimalPlaces = 3, generates "#.000" etc... 
        /// </summary>
        public string PriceChartTextFormatting
        {
            get
            {
                return _instrument.DecimalPlaces == 0 ? "#" : "0." + new string('0', _instrument.DecimalPlaces);
            }
        }

        private void InitializeChartSurface()
        {
            UpdateChartData(_instrument, _timeFrame);
            SetModifier(ModifierType.ZoomPan);
        }

        private void SetDefaults()
        {
            _instrument = _dataManager.AvailableInstruments.FirstOrDefault(x => x.Symbol == "INDU");
            _timeFrame = _dataManager.GetAvailableTimeFrames(_instrument).First();
            _chartType = ChartType.FastCandlestick;            
        }

        private void SetModifier(ModifierType modifierType)
        {
            ChartModifier = modifierType;
        }

        public ModifierType ChartModifier
        {
            get 
            {
                return _chartModifier;
            }
            set 
            {
                _chartModifier = value;
                OnPropertyChanged("ChartModifier");
            }
        }

        private void UpdateChartData(Instrument instrument, TimeFrame timeFrame)
        {
            // Get the data for the chart from the datasource
            var priceData = _dataManager.GetPriceData(instrument.Symbol, timeFrame);
            
            UpdatePriceChart(priceData);
            UpdateVolumeChart(priceData);

            _priceDataSet.InvalidateParentSurface(RangeMode.ZoomToFit);
            _volumeData.InvalidateParentSurface(RangeMode.ZoomToFit);
        }

        private void UpdatePriceChart(PriceSeries priceData)
        {
            // Clear existing chart series
            var priceDataset = new DataSeriesSet<DateTime, double>();

            // Create a new series and append Open, High, Low, Close data                
            _priceSeries = new OhlcDataSeries<DateTime, double>();
            priceDataset.Add(_priceSeries);
            _priceSeries.SeriesName = priceData.Symbol;
            _priceSeries.Append(priceData.TimeData, priceData.OpenData, priceData.HighData, priceData.LowData, priceData.CloseData);

            // Create a series for the 200 period SMA which will be plotted as a line chart
            _sma200Series = priceDataset.AddSeries();
            _sma200Series.SeriesName = "200 SMA";
            var ma200 = DataManager.Instance.ComputeMovingAverage(priceData.CloseData, 200);
            _sma200Series.Append(priceData.TimeData, ma200);

            // Create a series for the 50 period SMA which will be plotted as a line chart
            _sma50Series = priceDataset.AddSeries();
            _sma50Series.SeriesName = "50 SMA";
            var ma50 = DataManager.Instance.ComputeMovingAverage(priceData.CloseData, 50);
            _sma50Series.Append(priceData.TimeData, ma50);

            PriceData = priceDataset;

            // Update the chart type and timeframe with current settings
            UpdateChartType(_chartType);            
        }

        private void UpdateVolumeChart(PriceSeries prices)
        {
            // Clear existing chart series
            var volumeSet = new DataSeriesSet<DateTime, long>();

            // Create a new series and append Open, High, Low, Close data                
            var volumeSeries = volumeSet.AddSeries();
            volumeSeries.SeriesName = string.Format("{0} Volume", prices.Symbol);
            volumeSeries.Append(prices.TimeData, prices.VolumeData);

            VolumeData = volumeSet;
        }

        private void UpdateChartType(ChartType chartType)
        {
            if (PriceData.Count == 0)
            {
                UpdateChartData(_instrument, _timeFrame);
                return;
            }

            SelectedChartType = chartType;
        }

        public void OnExampleExit()
        {
            ChartModifier = ModifierType.Null;
            PriceData = null;
            VolumeData = null;
        }

        public void OnExampleEnter()
        {
            InitializeChartSurface();
        }
    }
}