﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Abt.Controls.SciChart.Example.MVVM;

namespace Abt.Controls.SciChart.Example.Helpers
{
    public class Navigator
    {
        #region Singleton

        private static Navigator _navigator;

        public static readonly DependencyProperty StartPageProperty = DependencyProperty.RegisterAttached(
            "StartPage",
            typeof (Guid),
            typeof (Navigator),
            new PropertyMetadata(OnStartPageAttached)
            );

        public static void SetStartPage(DependencyObject element, Guid value)
        {
            element.SetValue(StartPageProperty, value);
        }

        public static Guid GetStartPage(DependencyObject element)
        {
            return (Guid)element.GetValue(StartPageProperty);
        }

        private static void OnStartPageAttached(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var navigationFrame = (Frame) d;
            var page = GetStartPage(navigationFrame);

            CreateNavigationBase(navigationFrame);
            _navigator.Navigate(page);
        }

        private static void CreateNavigationBase(Frame navigationFrame)
        {
            Instance._navigationBase.Frame = navigationFrame;
            Instance._navigationBase.AfterNavigation = Instance.OnAfterNavigation;
            Instance._navigationBase.BeforeNavigation = Instance.OnBeforeNavigation;
        }

        public static Navigator Instance
        {
            get
            {
                return _navigator ??
                       (_navigator = new Navigator());
            }
        }

        #endregion

        private class ApplicationPageWrapper
        {
            public ApplicationPage Page { get; set; }
            public UserControl View { get; set; }
        }

        private ApplicationPageWrapper _currentPage;
        private NavigationBase _navigationBase;

        public Action<Exception, Guid?> NavigationCallback { get; set; }

        public Action<UserControl, BaseViewModel> BeforeNavigation { get; set; }

        public Action<UserControl, BaseViewModel> AfterNavigation { get; set; }

        private Navigator()
        {
            _navigationBase = new NavigationBase(null);
        }

        public void Navigate(Guid pageId)
        {
            _navigationBase.Navigate(NavigationBase.NavigationMode.Page,
                                     new Uri(Module.Register[pageId].Uri, UriKind.RelativeOrAbsolute));
        }

        public void Navigate(Func<Example, bool> predicate)
        {
            var example =
                Module.Examples.FirstOrDefault(predicate);

            if (example != null)
            {
                _navigationBase.Navigate(NavigationBase.NavigationMode.Page,
                                         new Uri(Module.Register[example.PageId].Uri, UriKind.RelativeOrAbsolute));
            }
        }

        public void GoBack()
        {
            _navigationBase.Navigate(NavigationBase.NavigationMode.Back, null);
        }

        public void GoForward()
        {
            _navigationBase.Navigate(NavigationBase.NavigationMode.Forward, null);
        }

        public bool CanGoBack()
        {
            return _navigationBase.CanGoBack();
        }

        public bool CanGoForward()
        {
            return _navigationBase.CanGoForward();
        }

        public ApplicationPage GetCurrentPage()
        {
            var uri = _navigationBase.GetCurrentSource();

            if (uri == null)
                return null;

            return Module.Register.Where(item => item.Value.Uri == uri.OriginalString)
                .Select(item => item.Value)
                .FirstOrDefault();
        }

        public bool CanNavigateTo(Guid pageId)
        {
            return _navigationBase.CanNavigateTo(Module.Register[pageId].Uri);
        }

        private void OnBeforeNavigation()
        {
            if (_currentPage != null)
            {
                var viewModel =  _currentPage.Page != null ? _currentPage.Page.ViewModel : null;
                BeforeNavigation(_currentPage.View, viewModel);
            }
        }

        private void OnAfterNavigation(Exception e, UserControl view)
        {
            _currentPage = _currentPage ?? (new ApplicationPageWrapper());
            _currentPage.View = view;

            if (view != null)
            {   
                _currentPage.Page = GetCurrentPage();
                _currentPage.Page.NewViewModel();
                AfterNavigation(_currentPage.View, _currentPage.Page.ViewModel);

                NavigationCallback(e, _currentPage.Page.PageId);
            }
            else
            {
                _currentPage.Page = null;

                NavigationCallback(e, null);
            }
        }
    }
}
