﻿using System;
using System.Collections.Generic;
using System.Timers;
using Abt.Controls.SciChart.Example.Common;
using Abt.Controls.SciChart.Example.Data;
using Abt.Controls.SciChart.Example.MVVM;
using Abt.Controls.SciChart.Numerics;

namespace Abt.Controls.SciChart.Example.Examples.IWantTo.SeeFeaturedApplication.Oscilloscope
{
    public class OscilloscopeViewModel : BaseViewModel, IExampleAware
    {
        private DoubleRange _xVisibleRange;
        private DoubleRange _yVisibleRange;
        private ModifierType _chartModifier;
        private bool _isDigitalLine = true;
        private IDataSeriesSet<double, double> _dataSet;
        private IXyDataSeries<double, double> _series0;
        private Timer _timer;
        private double _phase0 = 0.0;
        private double _phase1 = 0.0;
        private double _phaseIncrement;
        private ResamplingMode _resamplingMode;
        private bool _canExecuteRollover;

        private string _selectedDataSource;

        private const double TimerIntervalMs = 20;

        public OscilloscopeViewModel()
        {
            // For chart data setup, see  OnExampleEnter(), which is called by the SciChart Examples Suite
        }

        public ActionCommand SetRolloverModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.Rollover)); } }
        public ActionCommand SetCursorModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.CrosshairsCursor)); } }
        public ActionCommand SetRubberBandZoomModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.RubberBandZoom)); } }
        public ActionCommand SetZoomPanModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.ZoomPan)); } }
        public ActionCommand SetNullModifierCommand { get { return new ActionCommand(() => SetModifier(ModifierType.Null)); } }
        public ActionCommand SetDigitalLineCommand { get { return new ActionCommand(() => IsDigitalLine = !IsDigitalLine); } }
        public ActionCommand ResetZoomCommand { get { return new ActionCommand(ResetZoom);}}

        public bool IsRolloverSelected { get { return ChartModifier == ModifierType.Rollover; }}
        public bool IsCursorSelected { get { return ChartModifier == ModifierType.CrosshairsCursor; } }

        public bool CanExecuteRollover
        {
            get { return _canExecuteRollover; }
            set
            {
                if (_canExecuteRollover == value) return;
                _canExecuteRollover = value;
                OnPropertyChanged("CanExecuteRollover");
            }
        }
        
        public IDataSeriesSet<double, double> ChartData
        {
            get { return _dataSet; }
            set
            {
                _dataSet = value;
                OnPropertyChanged("ChartData");
            }
        }

        public string SelectedDataSource
        {
            get { return _selectedDataSource; }
            set
            {
                if (_selectedDataSource == value) return;
                _selectedDataSource = value;

                lock (this)
                {
                    if (_selectedDataSource == "Lissajous")
                    {
                        // For Lissajous plots, we must use an UnsortedXyDataSeries 
                        // and we cannot use the Rollover. Currently HitTest/Rollover is not implemented
                        // for UnsortedXyDataSeries. Also this series type does not currently support resampling
                        _phaseIncrement = Math.PI * 0.02;
                        _series0 = new UnsortedXyDataSeries<double, double>();
                        IsDigitalLine = false;
                        if (ChartModifier == ModifierType.Rollover)
                            SetModifier(ModifierType.CrosshairsCursor);
                        CanExecuteRollover = false;
                        SeriesResamplingMode = ResamplingMode.None;                        
                    }
                    else
                    {
                        // For FourierSeries plots, we can use the faster sorted XyDataSeries, 
                        // which supports the Rollover, HitTest and Resamplingd
                        _phaseIncrement = Math.PI * 0.1;
                        _series0 = new XyDataSeries<double, double>();
                        IsDigitalLine = true;
                        CanExecuteRollover = true;
                        SeriesResamplingMode = ResamplingMode.MinMax;
                    }

                    ResetZoom();

                    // Add the new dataseries and reset counters. See OnTick where data is appended
                    ((IDataSeriesSet)ChartData).Clear();
                    ChartData.Add(_series0);
                    _series0.SeriesName = _selectedDataSource;
                    _series0.Clear();
                    _phase0 = 0;
                    _phase1 = 0.15;
                }
                OnPropertyChanged("SelectedDataSource");
            }
        }

        public bool IsDigitalLine
        {
            get { return _isDigitalLine; }
            set
            {
                if (_isDigitalLine == value) return;

                _isDigitalLine = value;
                OnPropertyChanged("IsDigitalLine");
            }
        }

        public DoubleRange XVisibleRange
        {
            get { return _xVisibleRange; }
            set 
            { 
                if (_xVisibleRange == value) return;

                _xVisibleRange = value;
                OnPropertyChanged("XVisibleRange");
            }
        }

        public DoubleRange YVisibleRange
        {
            get { return _yVisibleRange; }
            set
            {
                if (_yVisibleRange == value) return;

                _yVisibleRange = value;
                OnPropertyChanged("YVisibleRange");
            }
        }

        public ModifierType ChartModifier
        {
            get
            {
                return _chartModifier;
            }
            set
            {
                _chartModifier = value;
                OnPropertyChanged("ChartModifier");
                OnPropertyChanged("IsRolloverSelected");
                OnPropertyChanged("IsCursorSelected");
            }
        }

        public ResamplingMode SeriesResamplingMode
        {
            get { return _resamplingMode; }
            set
            {
                if (_resamplingMode == value) return;
                _resamplingMode = value;
                OnPropertyChanged("SeriesResamplingMode");
            }
        }
        

        private void SetModifier(ModifierType modifierType)
        {
            ChartModifier = modifierType;
        }

        // Reset state when example exits
        public void OnExampleExit()
        {
            lock (this)
            {
                if (_timer != null)
                {
                    _timer.Stop();
                    _timer.Elapsed -= OnTick;
                    _timer = null;
                }
                ChartData = null;
            }
        }

        // Setup start condition when the example enters
        public void OnExampleEnter()
        {
            ChartData = new DataSeriesSet<double, double>();

            SelectedDataSource = "Fourier Series";
            SetModifier(ModifierType.CrosshairsCursor);

            _timer = new Timer(TimerIntervalMs) { AutoReset = true };
            _timer.Elapsed += OnTick;
            _timer.Start();
        }

        private void OnTick(object sender, EventArgs e)
        {
            lock (this)
            {
                // Generate data at this phase depending on data source type
                var dataSource = SelectedDataSource == "Lissajous"
                                     ? DataManager.Instance.GetLissajousCurve(0.12, _phase1, _phase0, 2500)
                                     : DataManager.Instance.GetFourierSeries(2.0, _phase0, 1000);

                _phase0 += _phaseIncrement;
                _phase1 += _phaseIncrement*0.005;

                // Lock the dataset and clear / re-add new data
                using (this.ChartData.SuspendUpdates())
                {
                    _series0.Clear();
                    _series0.Append(dataSource.XData, dataSource.YData);
                }
            }
        }

        // Called by ResetZoomCommand, bound to DoubleClickCommandModifier (a custom modifier for this example)
        private void ResetZoom()
        {
            if (_selectedDataSource == "Lissajous")
            {
                XVisibleRange = new DoubleRange(-1.2, 1.2);
                YVisibleRange = new DoubleRange(-1.2, 1.2);
            }
            else
            {
                XVisibleRange = new DoubleRange(2.5, 4.5);
                YVisibleRange = new DoubleRange(-12.5, 12.5);
            }
        }
    }
}
