﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Timers;
using Abt.Controls.SciChart;

namespace OscilloscopeStyleChart
{
    public class OscilloscopeExampleViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        private IDataSeriesSet<double, double> _dataSet;
        private DoubleRange _xVisibleRange;
        private Timer _timer;

        private const double TimerIntervalMs = 5;
        private const double Dt = 1;

        private double t = 0.0;
        private IDataSeries<double, double> _series0;
        private double _xMajorDelta;
        private double _xMinorDelta;

        private double _lastXValue;
        private DoubleRange _yVisibleRange;
        private double _yMajorDelta;
        private double _yMinorDelta;
        private double _timeBase;
        private double _voltsPerDiv;

        private Random _random = new Random();
        private bool _isStepLine;

        public OscilloscopeExampleViewModel()
        {
            _dataSet = new DataSeriesSet<double, double>();
            _series0 = _dataSet.AddSeries();

            TimeBase = 5.0;
            VoltsPerDiv = 2.0;
            IsStepLine = false;

            _timer = new Timer(TimerIntervalMs) { AutoReset = true };
            _timer.Elapsed += TimerElapsed;
            _timer.Start();
        }

        /// <summary>
        /// SciChartSurface.DataSet binds to this
        /// </summary>
        public IDataSeriesSet<double, double> DataSet
        {
            get { return _dataSet; }
            set
            {
                _dataSet = value;
                OnPropertyChanged("DataSet");
            }
        }

        public double XMajorDelta
        {
            get { return _xMajorDelta; }
            set
            {
                _xMajorDelta = value;
                OnPropertyChanged("XMajorDelta");
            }
        }

        public double XMinorDelta
        {
            get { return _xMinorDelta; }
            set
            {
                _xMinorDelta = value;
                OnPropertyChanged("XMinorDelta");
            }
        }

        public DoubleRange XVisibleRange
        {
            get { return _xVisibleRange; }
            set
            {
                _xVisibleRange = value;
                OnPropertyChanged("XVisibleRange");
            }
        }

        public double YMajorDelta
        {
            get { return _yMajorDelta; }
            set
            {
                _yMajorDelta = value;
                OnPropertyChanged("YMajorDelta");
            }
        }

        public double YMinorDelta
        {
            get { return _yMinorDelta; }
            set
            {
                _yMinorDelta = value;
                OnPropertyChanged("YMinorDelta");
            }
        }

        public DoubleRange YVisibleRange
        {
            get { return _yVisibleRange; }
            set
            {
                _yVisibleRange = value;
                OnPropertyChanged("YVisibleRange");
            }
        }

        public IEnumerable<double> AllTimeBases
        {
            get { return new[] {0.1, 0.5, 1.0, 2.0, 5.0, 10.0}; }
        }

        public IEnumerable<double> AllVoltsPerDiv
        {
            get { return new[] {0.5, 1.0, 2.0, 5.0}; }
        }

        public double TimeBase
        {
            get { return _timeBase; }
            set
            {
                _timeBase = value;
                UpdateTimeBase(_timeBase);
                OnPropertyChanged("TimeBase");
            }
        }

        public double VoltsPerDiv
        {
            get { return _voltsPerDiv; }
            set
            {
                _voltsPerDiv = value;
                UpdateVoltage(_voltsPerDiv);
                OnPropertyChanged("VoltsPerDiv");
            }
        }        

        public bool IsStepLine
        {
            get { return _isStepLine; }
            set
            {
                _isStepLine = value;
                OnPropertyChanged("IsStepLine");
            }
        }

        public void OnPropertyChanged(string prop)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(prop));
            }
        }

        private void UpdateTimeBase(double msPerDiv)
        {
            // For N µs/div, choose MajorDelta = N (ensures one major tick every N µs)
            // Minor Delta = N/5 (ensures 5 minor ticks per major tick)
            // VisibleRange.Min = 0
            // Visiblerange.Max = 10*N (ensures 10 major divisions on the X-Axis chart)
            XMajorDelta = msPerDiv;
            XMinorDelta = msPerDiv*0.5;
            XVisibleRange = new DoubleRange(0, 10*XMajorDelta);
        }

        private void UpdateVoltage(double voltsPerDiv)
        {
            YMajorDelta = voltsPerDiv;
            YMinorDelta = voltsPerDiv*0.5;
            YVisibleRange = new DoubleRange(-5*voltsPerDiv, 5*voltsPerDiv);
        }

        private void TimerElapsed(object sender, ElapsedEventArgs e)
        {
            lock (this)
            {
                AppendPoint();
            }            
        }

        private void AppendPoint()
        {
            // Generate Y-Value from time, t
            double voltage = 5.0 * Math.Sin(2 * Math.PI * t / 20) +
                            2.0 * Math.Sin(2 * Math.PI * t / 10) +
                            1.0 * _random.NextDouble();

            // Generate X-Value, once the trace touches the right hand of the screen, 
            // we want to reset to the left
            double xValue = t % (XVisibleRange.Max + 1);

            // If a reset occurred, clear the series before adding the point
            if (xValue < _lastXValue)
            {
                _series0.Clear();
            }

            // Append x,y data
            _series0.Append(xValue, voltage);

            // Increment sample time
            t += Dt;
            _lastXValue = xValue;
        }
    }
}
