﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Timers;
using Abt.Controls.SciChart;

namespace ECGstyleChart
{
    public class ECGExampleViewModel : INotifyPropertyChanged    
    {
        private IDataSeriesSet<double, double> _dataSet;
        public event PropertyChangedEventHandler PropertyChanged;

        private Timer _timer;
        private IDataSeries<double, double> _series0;
        private double[] _sourceData;
        private int _currentIndex;
        private int _totalIndex;
        private DoubleRange _xVisibleRange;        
        private DoubleRange _yVisibleRange;
        private bool _isBeat;
        private int _heartRate;
        private bool _lastBeat;
        private DateTime _lastBeatTime;

        private const double WindowSize = 5.0;
        private const int TimerInterval = 20;

        public ECGExampleViewModel()
        {
            _dataSet = new DataSeriesSet<double, double>();

            // Create a data series. We use FIFO series as we
            // want to discard old data after 5000pts
            _series0 = _dataSet.AddFifoSeries(5000);

            // Simulate waveform
            _sourceData = LoadWaveformData("Waveform.txt");

            // Fix chart range in Y-Direction
            YVisibleRange = new DoubleRange(-0.5, 1.5);

            _timer = new Timer(TimerInterval) { AutoReset = true };
            _timer.Elapsed += TimerElapsed;
            _timer.Start();            
        }       

        /// <summary>
        /// SciChartSurface.DataSet binds to this
        /// </summary>
        public IDataSeriesSet<double, double> DataSet
        {
            get { return _dataSet; }
            set
            {
                _dataSet = value;
                OnPropertyChanged("DataSet");
            }
        }

        /// <summary>
        /// SciChartSurface.YAxis.VisibleRange binds to this
        /// </summary>
        public DoubleRange YVisibleRange
        {
            get { return _yVisibleRange; }
            set
            {
                _yVisibleRange = value;
                OnPropertyChanged("YVisibleRange");
            }
        }

        /// <summary>
        /// SciChartSurface.XAxis.VisibleRange binds to this
        /// </summary>
        public DoubleRange XVisibleRange
        {
            get { return _xVisibleRange; }
            set
            {
                _xVisibleRange = value;
                OnPropertyChanged("XVisibleRange");
            }
        }

        /// <summary>
        /// The heartbeat graphic binds to this, and changes its scale on heartbeat 
        /// </summary>
        public bool IsBeat
        {
            get { return _isBeat; }
            set
            {
                _isBeat = value;
                OnPropertyChanged("IsBeat");
            }
        }

        /// <summary>
        /// The heartrate textblock binds to this
        /// </summary>
        public int HeartRate
        {
            get { return _heartRate; }
            set 
            { 
                _heartRate = value;
                OnPropertyChanged("HeartRate");
            }
        }

        public void OnPropertyChanged(string prop)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(prop));
            }
        }

        private void TimerElapsed(object sender, ElapsedEventArgs e)
        {
            lock (this)
            {
                // As timer cannot tick quicker than ~20ms, we append 10 points
                // per tick to simulate a sampling frequency of 500Hz (e.g. 2ms per sample)
                for (int i = 0; i < 10; i++)
                    AppendPoint(400);
            }
        }

        private void AppendPoint(double sampleRate)
        {
            if (_currentIndex >= _sourceData.Length)
            {
                _currentIndex = 0;
            }

            // Get the next voltage and time, and append to the chart
            double voltage = _sourceData[_currentIndex];
            double time = _totalIndex / sampleRate;
            _series0.Append(time, voltage);

            // Calculate the next visible range
            XVisibleRange = ComputeXAxisRange(time);

            IsBeat = voltage > 0.5;

            if (IsBeat && !_lastBeat)
            {
                HeartRate = (int)(60.0 / (DateTime.Now - _lastBeatTime).TotalSeconds);
                _lastBeatTime = DateTime.Now;
            }

            _lastBeat = IsBeat;
            _currentIndex++;
            _totalIndex++;
        }

        private static DoubleRange ComputeXAxisRange(double t)
        {
            if (t < WindowSize)
            {
                return new DoubleRange(0, WindowSize);
            }

            // Calculates a visible range. When the trace touches the right edge of the chart
            // (governed by WindowSize), shift the entire range 50% so that the trace is in the 
            // middle of the chart 
            double fractionSize = WindowSize * 0.5;
            double newMin = fractionSize * Math.Floor((t - fractionSize) / fractionSize);
            double newMax = newMin + WindowSize;

            return new DoubleRange(newMin, newMax);
        }

        private double[] LoadWaveformData(string filename)
        {
            var values = new List<double>();
            var asm = Assembly.GetExecutingAssembly();
            var resourceString = asm.GetManifestResourceNames().Single(x => x.Contains(filename));

            using (var stream = asm.GetManifestResourceStream(resourceString))
            using (var streamReader = new StreamReader(stream))
            {
                string line = streamReader.ReadLine();
                while (line != null)
                {
                    values.Add(double.Parse(line, NumberFormatInfo.InvariantInfo));
                    line = streamReader.ReadLine();
                }
            }

            return values.ToArray();
        }
    }
}
