﻿using System;
using System.Timers;
using System.Windows;
using System.Windows.Threading;
using Abt.Controls.SciChart;

namespace SimpleLineChart
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // Data Sample Rate (sec)  - 50 Hz
        private double dt = 0.02;

        // FIFO Size is 200 samples, meaning after 200 samples have been appended, each new sample appended
        // results in one sample being discarded
        private int FifoSize = 200;

        // Timer to process updates
        private Timer _timerNewDataUpdate;

        // The current time
        private double t;

        // The dataset to fill 
        private IDataSeriesSet<double, double> dataset;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            ResetButton_Click(this, null);
            StartButton_Click(this, null);
        }

        private void CreateDataSetAndSeries()
        {
            // Create a new dataset of type X=double, Y=double
            dataset = new DataSeriesSet<double, double>();

            if (isFifoCheckBox.IsChecked == true)
            {
                // Add three FIFO series to fill with data.                 
                // calling dataset.AddFifoSeries(int size) will create a FIFO series. New data is appended until the size is met, at which point
                //  old data is discarded. Internally the FIFO series is implemented as a circular buffer so that old data is pushed out of the buffer
                //  once the capacity has been reached
                // Note: Once a FIFO series has been added to a dataset, all subsequent series must be FIFO series. In addition, the FifoSize must be the
                //  same for all FIFO series in a dataset. 
                dataset.AddFifoSeries(FifoSize);
                dataset.AddFifoSeries(FifoSize);
                dataset.AddFifoSeries(FifoSize);
            }
            else
            {
                // Add three standard series to fill with data.            
                // calling dataset.AddSeries() will create a normal dataseries. New data is appended and no old data is discarded
                dataset.AddSeries();
                dataset.AddSeries();
                dataset.AddSeries();
            }

            // Set the dataset on the chart
            sciChartSurface.DataSet = dataset;
        }

        private void ClearDataSeries()
        {
            if (dataset == null)
                return;

            using (sciChartSurface.SuspendUpdates())
            {
                dataset[0].Clear();
                dataset[1].Clear();
                dataset[2].Clear();
            }
        }

        private void OnNewData(object sender, ElapsedEventArgs elapsedEventArgs)
        {
            // Compute our three series values
            double y1 = 3.0 * Math.Sin(((2 * Math.PI) * 1.4) * t);
            double y2 = 2.0 * Math.Cos(((2 * Math.PI) * 0.8) * t);
            double y3 = 1.0 * Math.Sin(((2 * Math.PI) * 2.2) * t);

            // Suspending updates is optional, and ensures we only get one redraw
            // once all three dataseries have been appended to
            using (sciChartSurface.SuspendUpdates())
            {
                // Get the series we previously created
                var series1 = dataset[0];
                var series2 = dataset[1];
                var series3 = dataset[2];

                // Append x,y data
                series1.Append(t, y1);
                series2.Append(t, y2);
                series3.Append(t, y3);
            }

            // Increment current time
            t += dt;
        }

        private void StartButton_Click(object sender, RoutedEventArgs e)
        {
            startButton.IsEnabled = false;
            pauseButton.IsEnabled = true;
            resetButton.IsEnabled = true;
            isFifoCheckBox.IsEnabled = false;

            CreateDataSetAndSeries();

            // Start a timer to create new data and append on each tick
            _timerNewDataUpdate = new Timer(dt*1000);
            _timerNewDataUpdate.Elapsed += OnNewData;
            _timerNewDataUpdate.AutoReset = true;
            _timerNewDataUpdate.Start();
        }

        private void PauseButton_Click(object sender, RoutedEventArgs e)
        {
            if (_timerNewDataUpdate != null)
            {
                _timerNewDataUpdate.Stop();
            }
            startButton.IsEnabled = true;
            pauseButton.IsEnabled = false;
            resetButton.IsEnabled = true;
            isFifoCheckBox.IsEnabled = false;
        }

        private void ResetButton_Click(object sender, RoutedEventArgs e)
        {
            PauseButton_Click(this, null);

            startButton.IsEnabled = true;
            pauseButton.IsEnabled = false;
            resetButton.IsEnabled = false;
            isFifoCheckBox.IsEnabled = true;

            t = 0.0;

            ClearDataSeries();
        }
    }
}

